package com.zbsz.service;

import com.zbsz.entity.Indicator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class DynamicTableService {
    @Resource
    private DataSource dataSource;

    private String getTableName(Indicator indicator){
        String tablename = "indicator_"+indicator.getSceneId()
                +"_"+indicator.getIndicatorDicId()
                +"_"+indicator.getAppDicPId()
                +"_"+indicator.getAppDicCId()
                +"_"+indicator.getIndicatorEnName()
                ;
        return tablename;
    }

    public boolean createTable(Indicator indicator) throws Exception{
        Connection conn = null;
        Statement stm = null;
        String tableName = null;
        try {
            tableName = getTableName(indicator);
            conn = dataSource.getConnection();
            stm = conn.createStatement();
            String exists_sql = "select exists (select from information_schema.tables where table_name = '"+tableName+"') as is_exists";
            String create_sql = "create table "+tableName+" (id int4 generated always as identity primary key,"+indicator.getIndicatorEnName()+" varchar(255) not null)";
            String alter_sql = "alter table "+tableName+" add column if not exists "+indicator.getIndicatorEnName()+" varchar(255)";
            log.info("---------- 查询表是否存在 sql："+exists_sql);
            ResultSet rs = stm.executeQuery(exists_sql);
            boolean flag = true;
            while(rs.next()){
                flag = rs.getBoolean("is_exists");
            }
            int result = 0;
            if(flag){
                log.info("---------- 创建表 sql："+exists_sql);
                result = stm.executeUpdate(create_sql);
            }else {
                log.info("---------- 添加表字段 sql："+exists_sql);
                result = stm.executeUpdate(alter_sql);
            }
            if(result>0){
                return true;
            }else {
                log.error("创建表，table name : "+tableName+"失败");
                return false;
            }
        }catch (Exception e){
            throw new Exception("创建动态表失败，table name : "+tableName,e);
        }finally {
            try {
                if(stm != null) stm.close();
            }catch (Exception e){
                throw new Exception("stm.close() 异常！");
            }
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new Exception("conn.close() 异常！");
            }
        }
    }

    public boolean deleteTable(Indicator indicator) throws Exception{
        Connection conn = null;
        Statement stm = null;
        String tableName = null;
        try {
            List<String> columnList = new ArrayList();
            tableName = getTableName(indicator);
            conn = dataSource.getConnection();
            stm = conn.createStatement();
            String exists_sql = "select exists (select from information_schema.tables where table_name = '"+tableName+"') as is_exists";
            ResultSet rs = stm.executeQuery(exists_sql);
            boolean flag = true;
            while(rs.next()){
                flag = rs.getBoolean("is_exists");
            }
            if(!flag){
                throw new Exception("删除动态表失败，table name : "+tableName+" 不存在！！！");
            }

            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet resultSet = metaData.getColumns(null,null,tableName,null);
            while (resultSet.next()) {
                columnList.add(resultSet.getString("COLUMN_NAME"));
            }

            String alter_sql = "alter table "+tableName+" drop column if exists "+indicator.getIndicatorEnName();
            String drop_sql = "drop table if exists"+tableName;

            int result = 0;
            if(columnList.size()>1){
                log.info("---------- 删除表字段 sql："+alter_sql);
                result = stm.executeUpdate(alter_sql);
            }else {
                log.info("---------- 删除表 sql："+drop_sql);
                result = stm.executeUpdate(drop_sql);
            }

            if(result>0){
                return true;
            }else {
                log.error("删除动态表失败，table name : "+tableName);
                return false;
            }
        }catch (Exception e){
            throw new Exception("删除动态表失败，table name : "+tableName,e);
        }finally {
            try {
                if(stm != null) stm.close();
            }catch (Exception e){
                throw new Exception("stm.close() 异常！");
            }
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new Exception("conn.close() 异常！");
            }
        }
    }


}
