package com.zbsz.service;

import com.zbsz.entity.Indicator;
import com.zbsz.info.Search;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class SearchService {
    @Resource
    private DataSource dataSource;

    public List<String> getTableNames() throws Exception{
        Connection conn = null;
        try {
            List<String> list = new ArrayList();
            conn = dataSource.getConnection();
            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet resultSet = metaData.getTables(null, null, null, new String[]{"TABLE"});
            while (resultSet.next()) {
                list.add(resultSet.getString("TABLE_NAME"));
            }
            return list;
        }catch (Exception e){
            throw new Exception("获取数据库列表失败！",e);
        }finally {
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new Exception("获取数据库列表失败，conn.close() 异常！");
            }
        }
    }


    public List<String> getTableColumns(String tableName) throws Exception{
        Connection conn = null;
        try {
            List<String> list = new ArrayList();
            conn = dataSource.getConnection();
            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet resultSet = metaData.getColumns(null,null,tableName,null);
            while (resultSet.next()) {
                //1、COLUMN_NAME：column name and size  2、TYPE_NAME：data type name
                list.add(resultSet.getString("COLUMN_NAME"));
            }
            return list;
        }catch (Exception e){
            throw new Exception("获取数据库列表失败！",e);
        }finally {
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new Exception("获取数据库列表失败，conn.close() 异常！");
            }
        }
    }


    public List searchIndicator(Search search) throws Exception{
        Connection conn = null;
        Statement stm = null;
        //需要上一层设置
        String tableName = search.getTableName();
        try {
            conn = dataSource.getConnection();
            stm = conn.createStatement();
            String left_join_sql = "select * from "+tableName+" left join ";
            log.info("---------- 左连接合并表 sql："+left_join_sql);

        }catch (Exception e){
            throw new Exception("创建动态表失败，table name : "+tableName,e);
        }finally {
            try {
                if(stm != null) stm.close();
            }catch (Exception e){
                throw new Exception("stm.close() 异常！");
            }
            try {
                if(conn != null) conn.close();
            }catch (Exception e){
                throw new Exception("conn.close() 异常！");
            }
        }
        return null;
    }


}
